/*****************************************************************************
**  This is part of the CTSim program
**  Copyright (C) 1983-2000 Kevin Rosenberg
**
**  $Id: sgp.h,v 1.4 2000/04/30 19:17:35 kevin Exp $
**  $Log: sgp.h,v $
**  Revision 1.4  2000/04/30 19:17:35  kevin
**  Set up include files for conditional INTERACTIVE_GRAPHICS
**
**  Revision 1.3  2000/04/28 18:35:21  kevin
**  removed unused files
**
**  Revision 1.2  2000/04/28 14:14:16  kevin
**  *** empty log message ***
**
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License (version 2) as
**  published by the Free Software Foundation.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/
/*----------------------------------------------------------------------*/
/*		   Standard Graphics Package Header File		*/
/*----------------------------------------------------------------------*/

#ifndef __H_SGP
#define __H_SGP

#include "kstddef.h"
#include "kmath.h"

/* device names */

#define CRTDEV  1
#define PRTDEV  2
#define MEMDEV  4
#define FILEDEV 8

/* linestyles */

#define LS_NOLINE 0
#define LS_SOLID  0xffff
#define LS_DASH1  0xff00
#define LS_DASH2  0xf0f0
#define LS_DASH3  0xcccc
#define LS_DASH4  0xff3e
#define LS_DOTTED 0xaaaa

#define MAXDASH	   4
#define MAXCOLOR   63

/* data structures */

struct device_st {
	int open;			/* TRUE if device is open for output */
	int xsize, ysize;		/* Size of device in pixels */
	int xmin, ymin;			/* smallest coordinates */
	int xmax, ymax;			/* Maximum coordinates */
	int colormax;			/* Maximum color number of device */
	int style;			/* Current linestyle of device */
	int width;			/* Current width of device */
	int color;			/* Current color of device */
	int icurx, icury;		/* Current position */
	int icwidth, icheight;		/* Size of characters in pixels */
	int cfore, cback;		/* Character foregnd & backgnd colors */
	float asp;			/* Aspect ratio.  Multipy x coord */
	int (*dotfunc)(int x1, int y1, int color);		/* Dot function for device */
	int (*linefunc)(int x1, int y1, int x2, int y2, int color);		/* Line function for device */
	unsigned int nbytes;		/* Size of buffer in bytes */
	char *buf;			/* Pointer to buffer */
	unsigned int bufseg, bufoff;	/* Buffer memory location */
	int mode;			/* Device mode */
};

struct charsp_st {
	float width, height;		/* size of characters in NDC */
	float textangle;		/* text angle in radians */
	float charupangle;		/* character up angle */
	int font;			/* font for characters */
	int updir, textdir;		/* text direct & character orientation */
	int fore, back;			/* foreground & background color */
					/* if back = -1, then transparent back */
};

struct state_st {
	int foregnd, backgnd;		/* current foregound & background colors */
	int linestyle;			/* current 16 bit linestyle */
	int linewidth;			/* current width of line (in pixels) */
	int marktype;			/* current marker type */
	int markcolor;			/* current marker color */
	float xndc, yndc;		/* current position in NDC */
};

typedef struct device_st DEVICE;
typedef struct charsp_st CHARSPEC;
typedef struct state_st GRFSTATE;

struct point	{double x, y, z;};


/******************************************************************
 *								  *
 *   GRAPH.H							  *
 *								  *
 *   Max R. Dursteler	  Dec 1983				  *
 *   12405 Village Square Terrace				  *
 *   Rockville Md. 20852					  *
 ******************************************************************
 */

/* Constants */

/* Flagcodes for motion directions */
#define XPLUS	001	/* right */
#define XMINUS	002	/* left */
#define YPLUS	004	/* up */
#define YMINUS	010	/* down */

/* Codes for marker symbols */
#define POINT	 0	/* small dot */
#define SQUARE	 1	/* empty square */
#define FSQUARE  2	/* filled square */
#define DIAMOND  3	/* empty diamond */
#define FDIAMOND 4	/* filled diamond */
#define CROSS	 5	/* cross */
#define XCROSS	 6	/* x */
#define CERCLE	 7	/* open circle */
#define FCERCLE  8	/* filled circle */
#define BSQUARE  9	/* big open square */
#define BDIAMOND 10	/* big open diamond */

#define NMARKERS 11	/* Number of available symbol types */

/*-------------------------------------------------------------------------*/

#define PSET   0		/* codes for raster merging */
#define PRESET 1
#define OR     2
#define AND    3
#define XOR    4

/*-------------------------------------------------------------------------*/

#define X_STKMIN  10		/* joystick specific values */
#define X_STKMAX 313
#define Y_STKMIN  10
#define Y_STKMAX 313

#define X_LOCMAX  (X_STKMAX - X_STKMIN);	/* range from readloc() */
#define Y_LOCMAX  (Y_STKMAX - Y_STKMIN);	/*   = 0 to LOCMAX */

#define JOYSTK1   0		/* device names */
#define JOYSTK2   1

#define BUTT_DONE  2		/* Right joystick button to end a command */
#define BUTT_START 1		/* Left joystick button to start a command */
#define BUTT_BOTH  3		/* Both buttons pressed */

#define RL_ERROR  -1		/* readloc() return codes */
#define RL_OFF     0
#define RL_ON	   1

/*-------------------------------------------------------------------------*/

struct raster_st {
    int type;
    int xmin, ymin;
    int xmax, ymax;
};

typedef struct raster_st RASTER;

/*-------------------------------------------------------------------------*/


/* circle.c */
void circle(const double r);
void drawarc(double start, double stop, const double r);
/* ctm.c */
int ctm_xlat_pre_2(double x, double y);
int ctm_xlat_post_2(double x, double y);
int ctm_scale_pre_2(double sx, double sy);
int ctm_scale_post_2(double sx, double sy);
int ctm_rotate_pre_2(double theta);
int ctm_rotate_post_2(double theta);
int ctm_shear_pre_2(double shrx, double shry);
int ctm_shear_post_2(double shrx, double shry);
int xlat_gmtx_2(GRFMTX_2D m, double x, double y);
int scale_gmtx_2(GRFMTX_2D m, double sx, double sy);
int shear_gmtx_2(GRFMTX_2D m, double shrx, double shry);
int rotate_gmtx_2(GRFMTX_2D m, double theta);
int ident_gmtx_2(GRFMTX_2D m);
int mult_gmtx_2(GRFMTX_2D a, GRFMTX_2D b, GRFMTX_2D c);
int invert_gmtx_2(GRFMTX_2D a, GRFMTX_2D b);
double determ_gmtx_2(GRFMTX_2D a);
/* drawbox.c */
int drawbox(double xmin, double ymin, double xmax, double ymax);
/* sgp.c */
int gp_init_2(void);
int window2(double xmin, double ymin, double xmax, double ymax);
int window_2(double xmin, double ymin, double xmax, double ymax);
int viewprt2(double xmin, double ymin, double xmax, double ymax);
int viewport_2(double xmin, double ymin, double xmax, double ymax);
int framevpt(void);
int calc_wc_to_ndc(void);
int calc_ndc_to_mc(void);
int wc_to_ndc(double xw, double yw, double *xn, double *yn);
int ndc_to_wc(double xn, double yn, double *xw, double *yw);
int color(int icol);
int linestyle(int style);
int line_abs_2(double x, double y);
int lineabs2(double x, double y);
int move_abs_2(double x, double y);
int moveabs2(double x, double y);
int line_rel_2(double x, double y);
int linerel2(double x, double y);
int move_rel_2(double x, double y);
int moverel2(double x, double y);
int draw_text(char *message);
int drawtext(char *message);
int polylnabs2(double x[], double y[], int n);
int markabs2(double x, double y);
int markrel2(double x, double y);
int pntabs2(double x, double y);
int pntrel2(double x, double y);
int ctm_clr_2(void);
int ctm_get_2(GRFMTX_2D m);
int ctm_set_2(GRFMTX_2D m);
int ctm_pre_mult_2(GRFMTX_2D m);
int ctm_post_mult_2(GRFMTX_2D m);
/* sgpdrive.c */
int initgrf2(void);
int initdevice(int dev, int mode, int xsize, int ysize);
int opendevice(int dev);
int closedevice(int dev);
int termdevice(int dev);
void stylus(double x, double y, int beam);
int pntndc(double x, double y);
int markndc(double x, double y);
GRFSTATE *inqstate(void);
int gp_set_aspect(int dev, double asp);
void setlinestyle(int style);
int setlinewidth(int wid);
DEVICE *inqdev(int dev);
int settext(double width, double height, double textangle, int font);
int settextclr(int fore, int back);
int setcolor(int fore);
int setbackg(int back);
int initmarker(int marker, int color);
int settextdir(int direction);
int charsize(double wid, double height);
int textangle(double angle);
int drivtext(char *message);
int termgrf2(void);
int flushdevice(int dev);
/* sgptext.c */
void wrtsymbol(int sym, int x, int y, DEVICE *dev);
void wrtchar(int ch, int x, int y, CHARSPEC *cspec, DEVICE *dev);
void wrttext(char txtstr[], int x, int y, CHARSPEC *cspec, DEVICE *dev);
void crtcolor(int mode, int *f, int *b);


#endif
